/**
 *  Copyright (c) 2019 JSC electronics
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     https://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#include <ArduinoUnitTests.h>
#include "../src/SimpleRelay.h"

constexpr static byte INPUT_PIN = 10;
GodmodeState* state = GODMODE();

unittest_setup() {
    state->reset();
}

unittest(turn_on_non_inverted_relay) {
    SimpleRelay relay = SimpleRelay(INPUT_PIN);
    relay.on();

    assertEqual(HIGH, digitalRead(INPUT_PIN));
    assertEqual(true, relay.isRelayOn());
}

unittest(turn_on_inverted_relay) {
    SimpleRelay relay = SimpleRelay(INPUT_PIN, /* isInverted */ true);
    relay.on();

    assertEqual(LOW, digitalRead(INPUT_PIN));
    assertEqual(true, relay.isRelayOn());
}

unittest(turn_off_non_inverted_relay) {
    SimpleRelay relay = SimpleRelay(INPUT_PIN);
    relay.on();
    assertEqual(true, relay.isRelayOn());

    // Turn off
    relay.off();

    assertEqual(LOW, digitalRead(INPUT_PIN));
    assertEqual(false, relay.isRelayOn());
}

unittest(turn_off_inverted_relay) {
    SimpleRelay relay = SimpleRelay(INPUT_PIN, /* isInverted */ true);
    relay.on();
    assertEqual(true, relay.isRelayOn());

    // Turn off
    relay.off();

    assertEqual(HIGH, digitalRead(INPUT_PIN));
    assertEqual(false, relay.isRelayOn());
}

unittest(toggle_from_off_on_non_inverted_relay) {
    SimpleRelay relay = SimpleRelay(INPUT_PIN);
    assertEqual(false, relay.isRelayOn());

    relay.toggle();

    assertEqual(HIGH, digitalRead(INPUT_PIN));
    assertEqual(true, relay.isRelayOn());
}

unittest(toggle_from_on_non_inverted_relay) {
    SimpleRelay relay = SimpleRelay(INPUT_PIN);
    relay.on();
    assertEqual(true, relay.isRelayOn());

    relay.toggle();

    assertEqual(LOW, digitalRead(INPUT_PIN));
    assertEqual(false, relay.isRelayOn());
}

unittest(toggle_from_off_on_inverted_relay) {
    SimpleRelay relay = SimpleRelay(INPUT_PIN, /* isInverted */ true);
    assertEqual(false, relay.isRelayOn());

    relay.toggle();

    assertEqual(LOW, digitalRead(INPUT_PIN));
    assertEqual(true, relay.isRelayOn());
}

unittest(toggle_from_on_inverted_relay) {
    SimpleRelay relay = SimpleRelay(INPUT_PIN, /* isInverted */ true);
    relay.on();
    assertEqual(true, relay.isRelayOn());

    relay.toggle();

    assertEqual(HIGH, digitalRead(INPUT_PIN));
    assertEqual(false, relay.isRelayOn());
}

unittest_main()